class Fasgram {
    //コンストラクタ
    constructor(obj, texts) {
        this.all_texts = texts; //問題文を格納
        this.selector = obj.selector;  //問題文などを格納するセレクタ
        this.control_panel = obj.control_panel; //コントロールパネルを格納するセレクタ
        this.local_storage_key = obj.local_storage_key; //ローカルストレージのキー
        //効果音ファイル
        this.mp3 = {
            start: new Audio('./mp3/start.mp3'),
            correct: new Audio('./mp3/correct.mp3'),
            incorrect: new Audio('./mp3/incorrect.mp3'),
            cursor: new Audio('./mp3/cursor.mp3'),
            cancel: new Audio('./mp3/cancel.mp3'),
            closing: new Audio('./mp3/closing.mp3')
        }
        //トレーニングごとに加算されるスコア
        this.score = {
            description: obj.score_description,
            read_aloud: obj.score_read_aloud,
            filling: obj.score_filling,
            sorting: obj.score_sorting,
            selecting: obj.score_selecting,
            fail: obj.score_fail
        }
        this.score_by_id = {}; //idごとのスコア
        this.session_score = 0;  //セッションのスコア
        this.session_goal = obj.session_goal;  //このスコアに達した時点で終了画面に移動する
        //webstorageから学習記録を読み込みscore_by_idに配列として格納する
        if(localStorage.getItem(obj.local_storage_key) != null) {
            const text = localStorage.getItem(obj.local_storage_key);
            this.score_by_id = JSON.parse(text);
        } else {
        //学習記録が存在しない場合，初期化する
            for(let i = 0; i < this.all_texts.length; i++) {
                this.score_by_id[this.all_texts[i].id] = 0;
            }
        }
        console.log(this.score_by_id);
        //読み上げ機能のインスタンス
        this.uttr = new SpeechSynthesisUtterance();
        this.voices = window.speechSynthesis.getVoices();
        this.msg = {
            filling_instruction: '適切な英文になるように，語句を入力しなさい。',
            sorting_instruction: '適切な英文になるように，語句を順にクリックしなさい。',
            selecting_instruction: '適切な英文になるように，(　)に当てはまる語句を1つクリックしなさい。',
            read_aloud_instruction: '音声に合わせ，声に出して読みましょう。',
            session_closing: 'トレーニング終了です。',
            category_closing: 'すべての項目を習得しました。',
            starting: 'ボタンを押して、トレーニングを開始しましょう。',
            correct: '正解',
            incorrect: '不正解',
            answer_is: '正解は',
            new_item: '新しい表現の学習',
            btn_erase_record: '学習履歴の消去',
            confirm_erase_record: 'ブラウザに保存された学習記録を消去します。',
            btn_home: 'はじめに戻る',
            confirm_home: 'スタート画面に戻ります。',
            btn_next: '次に進む',
            notice_speech_sentence: '英文をクリックして音声を聞きましょう。'
        }
    }
    //オートモード　自動でトレーニングを行う
    async auto(filter) {
        const self = this;
        await fasgram.starting(); //スタート画面
        //トレーニングの実行
        while(caterogy_end_judge() == false) {
            //トレーニングする項目をランダムで選ぶ
            const k = Math.floor(Math.random() * self.all_texts.length);
            const text = self.all_texts[k];
            const score = self.score_by_id[text.id];
            if((score >= filter.description.min && score <= filter.description.max)
                && score < filter.id_goal) {
                await self.description(k);
                if(judging_by_score()) {
                    break;
                }
            }
            if((score >= filter.read_aloud.min && score <= filter.read_aloud.max)
                && score < filter.id_goal) {
                await self.read_aloud(k);
                if(judging_by_score()) {
                    break;
                }
            }
            if((score >= filter.filling.min && score <= filter.filling.max)
                && score < filter.id_goal) {
                await self.filling(k);
                if(judging_by_score()) {
                    break;
                }
            }
            if((score >= filter.sorting.min && score <= filter.sorting.max)
                && score < filter.id_goal) {
                await self.sorting(k);
                if(judging_by_score()) {
                    break;
                }
            }
            if((score >= filter.selecting.min && score <= filter.selecting.max)
                && score < filter.id_goal) {
                await self.selecting(k);
                if(judging_by_score()) {
                    break;
                }
            }
        }
        if(caterogy_end_judge() == true) {
            await self.category_closing();
        } else {
            await self.session_closing();
        }
        return;
        //スコアの判定
        function caterogy_end_judge() {
            //すべての項目のスコアがid_goalを上回るときtrueを返す
            let val = true;
            Object.values(self.score_by_id).forEach(function(value) {
                if(value < filter.id_goal) {
                    val = false;
                }
            });
            return val;
        }
        function judging_by_score() {
            //セッションのスコアが完了点に達したらtrueを返す
            if(self.session_score >= self.session_goal) {
                return true;
            } else {
                return false;
            }
        }
    }
    //穴埋め問題　引数 texts 問題文などを格納したオブジェクト
    async filling(k) {
        let self = this;
        const text = self.all_texts[k];
        //引数オブジェクトの中に必要な要素がなければトレーニングを実行しない
        if(!text.ja_sentence || !text.en_sentence
            || !text.fill_sentence || !text.fill_answer) {
                return false;
        }
        self.set_control_panel(); //コントロールパネルの設置
        self.progress_bar(); //プログレスバーの表示
        //画面の初期化とレイアウトの設定
        const content = '<p id="instruction" class="size-4">'
            + self.msg.filling_instruction
            + '</p>'
            + '<p id="jp-sentence" class="size-3"></p>'
            + '<p id="en-sentence" class="size-2"></p>'
            + '<div id="speech_sentence"></div>'
            + '<p id="comment-box" class="size-3"></p>'
            + '<div id="keyboard"></div>'
            + '<div id="btn-confirm"></div>';
        $(self.selector).html(content);
        //問題を表示する
        const ja_sentence = text.ja_sentence;
        const en_sentence = text.en_sentence;
        let fill_sentence = new String(text.fill_sentence);
        var answer = new String(text.fill_answer);
        $('#jp-sentence').text(ja_sentence); //日本文
        //英文の空欄部を下線に置換する
        var chars = ''; //解答から空白を除いた文字列
        let blank = '';
        const answer_length = answer.length; //解答の文字数を取得
        blank += '&ensp;<span>';
        var count = 0;
        for(let i = 0; i < answer_length; i++) {
            if(answer.charAt(i) != ' ') {
                blank += ' <span id="blank'+count+'" class="bold color-blue">_</span> ';
                count += 1;
            } else {
                blank += '</span>&emsp;<span>';
            }
        }
        blank += '</span>&ensp;';
        fill_sentence = fill_sentence.replace('#', blank);
        $('#en-sentence').html(fill_sentence); //英文
        self.set_speech_sentence('#speech_sentence',
            '#en-sentence',
            en_sentence);//読み上げ機能の設置
        const key_selectors = self.keyboard('#keyboard'); //ソフトウェアキーボード
        //イベントリスナーの設置
        count = 0;
        let input;
        let finished;
        await new Promise((resolve) => {
            //クリックのとき
            for(let i = 0; i < key_selectors.length; i++) {
                $(key_selectors[i]).on('click', function() {
                    $(this).effect("highlight",{ color: "#ccdeff"}, 250);
                    input = $(this).val(); //入力された文字を取得
                    finished = input_processing(input);
                    if(finished) {
                        resolve();
                    }
                });
            }
            //キーボードが押されたとき
            $(document).on('keydown', async function(e) {
                if(e.keyCode >= 65 && e.keyCode <= 90) {
                    input = String.fromCharCode(e.keyCode).toLowerCase();
                    finished = input_processing(input);
                }
                if(e.keyCode == 8) {
                    input = 'BS';
                    input_processing(input);
                }
                if(finished) {
                    resolve();
                }
            });
        });
        //正解不正解の判定
        if(chars == answer.replace(/ /g, '')) {
            self.score_transaction(text.id, self.score.filling);
            self.mp3.correct.play();
            const content = '<p><i class="fas fa-check color-red"></i>'
                +' '+self.msg.correct+'</p>';
            $('#comment-box').html(content);
        } else {
            self.score_transaction(text.id, self.score.fail);
            self.mp3.incorrect.play();
            const content = '<p><i class="fas fa-times-circle color-red"></i>'
                + ' '+self.msg.incorrect+'</p>'
                + '<p>'+self.msg.answer_is+' '
                + answer
                + '</p>';
            $('#comment-box').html(content);
        }
        //「次に進む」ボタンの表示
        $('#keyboard').hide();
        self.progress_bar();
        self.speech_sentence(en_sentence);
        await self.button_confirm('#btn-confirm');
        speechSynthesis.cancel();
        $(document).off(); //イベントリスナーの解除
        return true;
        //アルファベット入力時の処理
        function input_processing(input) {
            //通常の文字のとき
            if(input != 'BS') {
                    self.mp3.cursor.currentTime = 0;
                    self.mp3.cursor.play();
                    chars += input;
                    $('#blank' + count).text(' '+input+' ');
                    count += 1;
            //バックスペースのとき
            } else {
                if(count > 0) {
                    self.mp3.cancel.currentTime = 0;
                    self.mp3.cancel.play();
                    chars = chars.slice(0,-1);
                    count -= 1;
                    $('#blank' + count).text('_');
                }
            }
            if(chars.length == answer.replace(/ /g, '').length) {
                return true;
            }
        }
    }
    //並べかえ問題　引数 texts 問題文などを格納したオブジェクト
    async sorting(k) {
        let self = this;
        const text = self.all_texts[k];
        //引数オブジェクトの中に必要な要素がなければトレーニングを実行しない
        if(!text.ja_sentence || !text.en_sentence
            || !text.sort_sentence || !text.sort_option
            || !text.sort_answer) {
                return false;
        }
        self.set_control_panel(); //コントロールパネルの設置
        self.progress_bar(); //プログレスバーの表示
        //画面の初期化とレイアウトの設定
        let content = '<p id="instruction" class="size-4 color-gray">'
            + self.msg.sorting_instruction
            + '</p>'
            + '<p id="ja-sentence" class="size-3"></p>'
            + '<p id="en-sentence" class="size-2"></p>'
            + '<div id="speech_sentence"></div>'
            + '<p id="console"></p>'
            + '<p id="comment-box" class="size-3"></p>'
            + '<div id="btn-confirm"></div>';
        $(self.selector).html(content);
        //問題文の表示
        const en_sentence = text.en_sentence;
        const ja_sentence = text.ja_sentence;
        let sentence = text.sort_sentence;
        const option = text.sort_option;
        const answer =text.sort_answer;
        //選択肢のシャッフル
        for(let i = 0; i < 100; i++) {
            let a = Math.floor(Math.random() * option.length);
            let b = Math.floor(Math.random() * option.length);
            [option[a], option[b]] = [option[b], option[a]];
        }
        let blank = '';
        for(let i = 0; i < option.length; i++) {
            blank += ' <span id="blank'+i+'" class="bold color-blue">___</span> ';
        }
        //問題文を表示
        $('#ja-sentence').text(ja_sentence); //日本文
        sentence = sentence.replace('#',blank);        
        $('#en-sentence').html(sentence); //問題文
        self.set_speech_sentence('#speech_sentence',
            '#en-sentence',
            en_sentence);//読み上げ機能の設置
        //選択肢のボタンを表示
        for(let i = 0; i < option.length; i++) {
            content = '<button type="button"'
                + ' id="option' + i + '"'
                + ' class="btn-option size-1"'
                + ' value="' + i + '">'
                + option[i]
                + '</button>'
            $('#console').append(content);
        }
        //backspaceアイコンの表示
        content = '<button type="button" '
            + 'id="key-BS" class="btn-option size-1" '
            + 'value="BS">'
            + '<i class="fas fa-backspace"></i>'
            + '</button>';
        $('#console').append(content);
        //選択肢クリック時の処理
        let input_phrase = [];
        await new Promise((resolve) => {
            let seq = 0;
            let input = [];
            //選択肢のクリック
            for(let i = 0; i < option.length; i++) {
                $('#option'+i).on('click', function() {
                    self.mp3.cursor.currentTime = 0;
                    self.mp3.cursor.play();
                    input[seq] = $(this).val();  //クリックされたボタンの番号
                    input_phrase[seq] = option[input[seq]];  //クリックされた文字列
                    $('#blank'+seq).text(' '+input_phrase[seq]+' ');
                    seq += 1;
                    $(this).hide();
                    if(seq == option.length) {
                        resolve();
                    }
                });
            }
            //backspaceのクリック
            $('#key-BS').on('click', function() {
                self.mp3.cancel.currentTime = 0;
                self.mp3.cancel.play();
                if(seq > 0) {
                    seq -= 1;
                    content = ' ___ ';
                    $('#blank'+seq).text(content);
                    $('#option'+input[seq]).show();
                }
            });    
        });
        //正解不正解の判定
        $('#console').hide();
        if(input_phrase.join(' ') == answer) {
            self.score_transaction(text.id, self.score.sorting);
            self.mp3.correct.play();
            content = '<p><i class="fas fa-check color-red"></i>'
                +' '+self.msg.correct+'</p>';
            $('#comment-box').html(content);
        } else {
            self.score_transaction(text.id, self.score.fail);
            self.mp3.incorrect.play();
            content = '<p><i class="fas fa-times-circle color-red"></i>'
                + ' '+self.msg.incorrect+'</p>'
                + '<p>'+self.msg.answer_is+' '
                + answer
                + '</p>';
            $('#comment-box').html(content);
        }
        //「次に進む」ボタンの表示
        self.progress_bar();
        self.speech_sentence(en_sentence);
        await self.button_confirm('#btn-confirm');
        speechSynthesis.cancel();
        $(document).off(); //イベントリスナーの解除
        return true;
    }
    //語句選択問題引数 texts 問題文などを格納したオブジェクト
    async selecting(k) {
        let self = this;
        const text = self.all_texts[k];
        //引数オブジェクトの中に必要な要素がなければトレーニングを実行しない
        if(!text.en_sentence
            || !text.select_sentence || !text.select_option
            || !text.select_answer) {
                return false;
        }
        self.set_control_panel(); //コントロールパネルの設置
        self.progress_bar(); //プログレスバーの表示
        //画面の初期化とレイアウトの設定
        let content = '<p id="instruction" class="size-4 color-gray">'
        + self.msg.selecting_instruction
        + '</p>'
        + '<p id="en-sentence" class="size-2"></p>'
        + '<div id="speech_sentence"></div>'
        + '<div id="console"></div>'
        + '<p id="comment-box" class="size-3"></p>'
        + '<div id="btn-confirm"></div>';
        $(self.selector).html(content);
        const en_sentence = text.en_sentence;
        let sentence = text.select_sentence.replace('#','(&emsp;)');
        const option = text.select_option;
        const answer =text.select_answer;
        //選択肢のシャッフル
        for(let i = 0; i < 100; i++) {
            let a = Math.floor(Math.random() * option.length);
            let b = Math.floor(Math.random() * option.length);
            [option[a], option[b]] = [option[b], option[a]];
        }
        //問題文と選択肢の表示
        $('#en-sentence').html(sentence);
        self.set_speech_sentence('#speech_sentence',
            '#en-sentence',
            en_sentence);//読み上げ機能の設置
        //選択肢のボタン
        for(let i = 0; i < option.length; i++) {
            content = '<button type="button"'
                + ' id="option' + i + '"'
                + ' class="btn-option btn-half size-1"'
                + ' value="' + option[i] + '">'
                + (i + 1) + '. '
                + option[i]
                + '</button>'
            $('#console').append(content);
        }
        //イベントリスナーの設置
        let clicked;
        await new Promise((resolve) => {
            for(let i=0; i < option.length; i++){
                $('#option' + i).on('click', function() {
                    clicked = $(this).val();
                    resolve();
                });
            }
        });
        //正解不正解の判定
        $('#console').hide();
        if(clicked == answer) {
            //正解の場合の処理
            self.score_transaction(text.id, self.score.selecting);
            self.mp3.correct.play();
            content = '<p><i class="fas fa-check color-red"></i>'
                +' '+self.msg.correct+'</p>';
            $('#comment-box').html(content);
        } else {
            //不正解の処理
            self.score_transaction(text.id, self.score.fail);
            self.mp3.incorrect.play();
            content = '<p><i class="fas fa-times-circle color-red"></i>'
                + ' '+self.msg.incorrect+'</p>'
                + '<p>'+self.msg.answer_is+' '
                + answer
                + '</p>';
            $('#comment-box').html(content);
        }
        //「次に進む」ボタンの表示
        self.progress_bar();
        self.speech_sentence(en_sentence);
        await self.button_confirm('#btn-confirm');
        speechSynthesis.cancel();
        $(document).off(); //イベントリスナーの解除
        return true;
    }
    //学習項目の表示　引数 texts 問題文などを格納したオブジェクト
    async description(k) {
        let self = this;
        const text = self.all_texts[k];
        //引数オブジェクトの中に必要な要素がなければトレーニングを実行しない
        if(!text.id || !text.description) {
            return false;
        }
        self.set_control_panel(); //コントロールパネルの設置
        self.progress_bar(); //プログレスバーの表示
        //新出事項を知らせるメッセージ
        if(self.score_by_id[text.id] == 0) {
            const content = '<div id="new-item"><div>'
                + self.msg.new_item
                + '</div></div>';
            $(self.selector).html(content).hide().toggle("fade", 1200);
            setTimeout(() => {
                $(self.selector).toggle("fade",1200);
            },1500);
            await new Promise((resolve) => {
                setTimeout(() => {
                    resolve();
                },2400);
            });
        }
        const content = '<div id="description">'
            + text.description
            + '</div>'
            + '<div id="btn-confirm"></div>';
        $(self.selector).html(content).hide().toggle("fade", 800);
        await self.button_confirm('#btn-confirm'); //「次に進む」ボタンの表示
        self.score_transaction(text.id, self.score.description);
        self.progress_bar();
        self.mp3.correct.play();
        $(self.selector).toggle("fade");
        await new Promise((resolve) => {
            setTimeout(() => {
                resolve();
            },1200);
        });
        $(document).off(); //イベントリスナーの解除
        $(self.selector).empty().show(); //フェードアウトをリセットする
        return true;
    }
    //音読　引数 texts 問題文などを格納したオブジェクト
    async read_aloud(k) {
        let self = this;
        const text = self.all_texts[k];
        //引数オブジェクトの中に必要な要素がなければトレーニングを実行しない
        if(!text.ja_sentence || !text.en_sentence) {
            return false;
        }
        self.set_control_panel(); //コントロールパネルの設置
        self.progress_bar(); //プログレスバーの表示
        let content = '<div id="btn-home"></div>'
        + '<p id="instruction" class="size-4 color-gray">'
        + self.msg.read_aloud_instruction
        + '</p>'
        + '<p id="ja-sentence" class="size-3"></p>'
        + '<p id="en-sentence" class="size-2"></p>'
        + '<div id="speech_sentence"></div>'
        + '<div id="btn-confirm"></div>';
        $(self.selector).html(content);
        const en_sentence = text.en_sentence;
        const ja_sentence = text.ja_sentence;
        $('#ja-sentence').text(ja_sentence);
        $('#en-sentence').html(en_sentence);
        self.set_speech_sentence('#speech_sentence',
            '#en-sentence',
            en_sentence);//読み上げ機能の設置
        self.speech_sentence(en_sentence);
        await self.button_confirm('#btn-confirm'); //「次に進む」ボタンの表示
        speechSynthesis.cancel();
        self.score_transaction(text.id, self.score.read_aloud);
        self.progress_bar();
        $(document).off(); //イベントリスナーの解除
        return true;
    }
    //idごとのスコアの加算処理
    score_transaction(id, score = 0) {
        let self = this;
        if(!(id in self.score_by_id)) { //キーが存在しない場合に初期化する
            self.score_by_id[id] = 0;
        }
        self.score_by_id[id] += score; //スコアの加算または減算
        if(self.score_by_id[id] < 0 ) { //減算によりマイナスになったら1にする
            self.score_by_id[id] = 1;
        }
        self.session_score += score; //セッションのスコアを加算または減算
        if(self.session_score < 0) { //減算によりマイナスになったら0にする
            self.session_score = 0;
        }
        //LocalStorageにスコア情報を保存
        const text = JSON.stringify(self.score_by_id); //オブジェクトをJSON文字列に変換
        localStorage.setItem(self.local_storage_key, text);
        console.log('id:'+id+' score:'+self.score_by_id[id]);
    }
    //idごとのスコアの取得
    get_score_by_id() {
        if(this.score_by_id != false) {
            return this.get_score_by_id;
        }
    }
    //セッションのスコアの取得
    get_session_score() {
        return this.session_score;
    }
    //スタート画面   
    async starting() {
        let self = this;
        //コントロールパネルの設置
        let content = '<button type="button" id="reset"'
            + ' class="btn-home size-4">'
            + '<i class="fas fa-exclamation-triangle"></i>'
            + ' '+self.msg.btn_erase_record
            + '</button>'
        $(self.control_panel).html(content);
        $('#reset').on('click', function(){
            const result = window.confirm(self.msg.confirm_erase_record);
            if(result) {
                localStorage.removeItem(self.local_storage_key);
                //学習記録が存在しない場合，初期化する
                for(let i = 0; i < self.all_texts.length; i++) {
                    self.score_by_id[self.all_texts[i].id] = 0;
                }
                console.log(this.score_by_id);
                location.reload();
            }
        });
    //スタートメッセージの表示
    content = '<p class="size-2">'
            + self.msg.starting
            + '</p>'
            + '<div class="btn-wrapper">'
            + '<button type="button" id="start-button"'
            + ' class="btn-start size-3">start'
            + '</button>'
            + '</div>';
        $(self.selector).html(content);
        await new Promise((resolve) => {
            $('#start-button').on('click', function() {
                self.mp3.start.play();
                $(this).effect("highlight",{ color: "#ccdeff"}, 500);
                $(self.selector).toggle("fade")
                setTimeout(() => {
                    resolve();
                },1200);
            });
            $(document).on('keydown', function(e) {
                if(e.keyCode == 13) {
                    $(self.control_panel).html('');
                    self.mp3.start.play();
                    $(self.selector).toggle("fade")
                    setTimeout(() => {
                        resolve();
                    },1000);
                }
            });
        });
        $(document).off(); //イベントリスナーの解除
        $(self.selector).empty().show(); //フェードアウトをリセットする
    }
    //セッションの終了
    async session_closing() {
        let self = this;
        const content = '<p class="size-2">'
            + self.msg.session_closing
            + '</p>'
            + '<div class="btn-wrapper">'
            + '<button type="button" id="closing" class="btn-confirm size-3">'
            + self.msg.btn_home
            + '</button>'
            + '</div>';
        $(self.selector).html(content).hide().toggle("fade");
        self.mp3.closing.play();
        await new Promise((resolve) => {
            //ボタンが押されたらスタート画面へ
            $('#closing').on('click', function() {
                resolve();
            });
            $(document).on('keydown', function(e) {
                if(e.keyCode == 13) {
                    resolve();
                }
            });
        });
    }
    //カテゴリーの終了
    async category_closing() {
        let self = this;
        const content = '<p class="size-2">'
            + self.msg.category_closing
            + '</p>'
            + '<div class="btn-wrapper">'
            + '<button type="button" id="closing" class="btn-confirm size-3">'
            + self.msg.btn_home
            + '</button>'
            + '</div>';
        $(self.selector).html(content).hide().toggle("fade");
        self.mp3.closing.play();
        await new Promise((resolve) => {
            //ボタンが押されたらスタート画面へ
            $('#closing').on('click', function() {
                resolve();
            });
            $(document).on('keydown', function(e) {
                if(e.keyCode == 13) {
                    resolve();
                }
            });
        });
    }
    //コントロールパネルの設置
    set_control_panel() {
        let self = this;
        const content = '<div id="progress-bar"></div>'
            + '<button type="button" id="home"'
            + ' class="btn-home size-4">'
            + '<i class="fas fa-home"></i>'
            + ' '+self.msg.btn_home
            + '</button>'
        $(self.control_panel).html(content);
        //最初に戻るボタンのクリック
        $('#home').on('click', function(){
            speechSynthesis.cancel(); //再生中の音声を停止
            const result = window.confirm(self.msg.confirm_home);
            if(result) {
                location.reload();
            }
        });
    }
    //進捗バーの表示
    progress_bar() {
        let self = this;
        const content = '<progress'
            + ' max="'+self.session_goal+'"'
            + ' value="'+self.session_score+'"></progress>'
        $('#progress-bar').html(content);
    }
    //次に進むボタン　引数 selector ボタンを格納するセレクタ
    async button_confirm(selector) {
        const content = '<div class="btn-wrapper">'
            + '<button type="button" id="confirm"'
            + ' class="btn-confirm size-3">'
            + '<i class="fas fa-caret-right"></i>'
            + ' '+this.msg.btn_next
            + '</button>'
            + '</div>';
        $(selector).html(content);
        await new Promise((resolve) => {
            $('#confirm').on('click', function() {
                resolve();
            })
            $(document).on('keydown', function(e) {
                if(e.keyCode == 13) {
                    resolve();
                }
            }); 
        });
        $(document).off(); //イベントリスナーの解除
    }
    //ソフトウェアキーボードの表示
    //引数 selector ソフトウェアキーボードを格納するセレクタ
    //戻り値：それぞれのボタンのセレクタを格納した配列
    keyboard(selector) {
        const key_set = [
            ['q','w','e','r','t','y','u','i','o','p'],
            ['a','s','d','f','g','h','j','k','l'],
            ['z','x','c','v','b','n','m','BS']
            ];
        let content = '';
        let selectors = [];
        for(let i = 0; i < key_set.length; i++) {
            content += '<div class="keyboard">';
            for(let j = 0; j < key_set[i].length; j++) {
                selectors.push('#key-' + key_set[i][j]);
                if(key_set[i][j] == 'BS'){
                    //backspaceアイコンの表示
                    content += '<button type="button" '
                    + 'id="key-BS" class="btn-key" '
                    + 'value="BS">'
                    + '<i class="fas fa-backspace"></i>'
                    + '</button>';
                } else {
                    //アルファベットの表示
                    content += '<button type="button" '
                        + 'id="key-' + key_set[i][j] + '" '
                        + 'class="btn-key" '
                        + 'value="' + key_set[i][j] + '">'
                        + key_set[i][j]
                        + '</button>';
                }
            }
            content += '</div>';
        }
        //キーボードの表示
        $(selector).html(content);
        return selectors;
    }
    //音声読み上げ機能の設置
    //引数　selector1 メッセージを格納するセレクタ
    //　　　selector2 クリック対象となるセレクタ
    //　　　en_sentence　読み上げる英文の文字列
    set_speech_sentence(selector1, selector2, en_sentence) {
        let self = this;
        //メッセージの表示
        const content = '<p class="color-gray size-4">'
            + '<i class="fas fa-volume-up"></i>'
            + self.msg.notice_speech_sentence+'</p>';
        $(selector1).html(content)
        //読み上げ機能の設置
        $(selector2).on('click',function() {
            $(this).effect("highlight", { color: "#ccdeff" }, 2000);
            self.speech_sentence(en_sentence);
        });
    }
    //音声読み上げ機能　引数 sentence 読み上げる英文の文字列
    speech_sentence(sentence) {
        speechSynthesis.cancel();
        const voices = speechSynthesis.getVoices();
        this.uttr.voice = voices[3]; //女性の声
        //this.uttr.voice = voices[5]; //男性の声
        this.uttr.volume = 1.0; //音量
        this.uttr.rate = 0.9; //読み上げ速度
        this.uttr.pitch = 1.0; //音程
        this.uttr.lang = 'en-US'; //言語
        this.uttr.text = sentence; //読み上げテキスト
        speechSynthesis.speak(this.uttr); //再生
    }
}